/*******************************************************************************/
/* This program is for demonstrating how institution and branch data can be    */ 
/* loaded for FinCEN regulatory reports.                                       */
/* The data should be adjusted to match with your organizational information.  */
/*                                                                             */
/* This has been revised to import filing institutions from csv file to        */
/* simplify the creation of multiple institutions per SAR requirements.        */
/* The list of branches are extracted from FSC_BRANCH_DIM and institutions     */
/* are mapped to branches, and institution data is then populated down to      */
/* their associated branches.                                                  */
/*******************************************************************************/
%macro rc_load_sample_location; 

/*******************************************************************************/
/* Sample transmitter data.                                                    */ 
/* Shared by all finCEN forms.                                                 */
/* There should be only one current record for each tax ID.                    */
/* The effective period should not have gaps or overlaps.                      */
/* Transmitter_key should be unique.                                           */
/*******************************************************************************/

data fincen_transmitter;
    if 0 then set rc_cfg.fincen_transmitter(%fcf_get_ds_opts(lib_name=rc_cfg, ds_name=fincen_transmitter));
    form_config_key=301;

    transmitter_key=1;
    transmitter_name='SAS Institute Inc.';                                                                                                    
    transmitter_address='100 SAS Campus Drive';                                                                                           
    transmitter_city_name='Cary';                                                                                               
    transmitter_state_code='NC';                                                                                              
    transmitter_zip_code='27513';                                                                                                
    transmitter_country_code='US' ;
    transmitter_contact_name='SAS TECHNICAL SUPPORT'; 
    transmitter_telephone='9196778008';                                                                                                                                                                                          
    transmitter_tax_id='111222333';                                                                                                
    transmitter_control_code='TBSATEST'; 
    effective_from_date=mdy(1,1,2000)*24*3600;
    effective_to_date=mdy(1,1,5999)*24*3600;
    entity_number=transmitter_tax_id;
    entity_name=transmitter_name;
    output;
    transmitter_key=2;
    form_config_key=302;
    transmitter_contact_name='SAS TECHNICAL SUPPORT CTR/DOEP'; 
    output;
    transmitter_key=3;
    form_config_key=303;
    transmitter_contact_name='SAS TECHNICAL SUPPORT CTR/DOEP'; 
    output;
    transmitter_key=4;
    form_config_key=312;
    transmitter_contact_name='SAS TECHNICAL SUPPORT CTR/DOEP'; 
    output;
      transmitter_key=5;
    form_config_key=311;
    transmitter_contact_name='SAS TECHNICAL SUPPORT'; 
    output;
run; 


%if &rc_debug = N %then
%do;
    /* Validate sample data before deleting and replacing. */
    %let forceReplaceYN=N;
    %global error_cnt;
    %let error_cnt=0;
    %let all_error_cnt=0;
    %rc_validate_institution_data(new_records=fincen_transmitter,
                                  record_type=transmitter,
                                  key_variable=transmitter_key,
                                  form_type=CTRX,
                                  form_config_key=312,
                                  form_type_code=);
    %let all_error_cnt = %eval ( &all_error_cnt. + &error_cnt.);
    %rc_validate_institution_data(new_records=fincen_transmitter,
                                  record_type=transmitter,
                                  key_variable=transmitter_key,
                                  form_type=SARX,
                                  form_config_key=311,
                                  form_type_code=);
    %let all_error_cnt = %eval ( &all_error_cnt. + &error_cnt.);
    %if (&trans_rc.=0 and &all_error_cnt= 0) or &forceReplaceYN.=Y %then %do;
	    proc sql;
           delete from rc_cfg.fincen_transmitter(%fcf_get_ds_opts(lib_name=rc_cfg, ds_name=fincen_transmitter));
        quit;
        proc append base=rc_cfg.fincen_transmitter(%fcf_get_ds_opts(lib_name=rc_cfg, ds_name=fincen_transmitter, output_ds=Y))
                    data=fincen_transmitter  force;
        run;
		
        %end;
     
%end;


/**********************************************************************************/
/* Sample institution data.                                                       */
/* This example shows that the institution information will be pulled from csv    */
/* file and can be different for each form type.                                  */
/*                                                                                */ 
/* There may also be more than one institution for a form type, such as SARX/SARX.*/
/* The effective period should not have gaps or overlaps.                         */
/* Institution_key should be unique in the csv file.                              */
/**********************************************************************************/

data fincen_institution (drop=effective_from_date_str 
                              effective_to_date_str);
      
     if 0 then set rc_cfg.fincen_institution(%fcf_get_ds_opts(lib_name=rc_cfg, ds_name=fincen_institution));

     length effective_from_date_str 
            effective_to_date_str $ 20
            ;
	 format effective_from_date datetime20.
	        effective_to_date   datetime20.
			;

     infile  rc_tmplt(rc_fincen_sample_institution_data.csv)  
             dsd lrecl=32764 truncover firstobs=2 delimiter=',' ;
     input 
          INSTITUTION_KEY                 $
          INSTITUTION_PRIM_FED_REG_CODE   $ 
          INSTITUTION_LEGAL_NAME          $
          INSTITUTION_TAX_ID              $
          INSTITUTION_TAX_ID_TYPE         $
          INSTITUTION_TYPE_CODE           $
          INSTITUTION_TYPE_OTHER          $
          INSTITUTION_SF_TYPE_CODE        $
          INSTITUTION_SF_TYPE_OTHER       $
          INSTITUTION_ID_TYPE_CODE        $  
          INSTITUTION_ID	              $	  
		  INSTITUTION_ADDRESS             $
		  INSTITUTION_CITY_NAME           $
		  INSTITUTION_STATE_CODE          $
		  INSTITUTION_ZIP_CODE            $
		  INSTITUTION_COUNTRY_CODE        $
		  INSTITUTION_ALTERNATE_NAME      $
		  CONTACT_OFFICE                  $
		  CONTACT_PHONE_NUMBER            $
		  CONTACT_PHONE_EXTENSION         $	  
          EFFECTIVE_FROM_DATE_STR         $ 
          EFFECTIVE_TO_DATE_STR           $       
          FORM_CONFIG_KEY                 	
          CONTACT_NAME                    $	
          CONTACT_TITLE                   $ 
		  INSTITUTION_GAMING_TYPE_CODE    $
		  INSTITUTION_GAMING_TYPE_OTHER   $
		  DEFAULT_IND                     $
		  ENTITY_NUMBER                   $
		  ENTITY_NAME                     $	  
    ;  
    effective_from_date=input(substr(effective_from_date_str,1,10),mmddyy10.)*24*3600;
    effective_to_date=input(substr(effective_to_date_str,1,10),mmddyy10.)*24*3600;     

run;


%if &rc_debug = N %then
%do;
    /* Validate sample data before deleting and replacing. */
    %let forceReplaceYN=N;
    %global error_cnt;
    %let error_cnt=0;
    %let all_error_cnt=0;
    %rc_validate_institution_data(new_records=fincen_institution,
                                  record_type=filer,
                                  key_variable=institution_key,
                                  form_type=CTRX,
                                  form_config_key=312,
                                  form_type_code=);
    %let all_error_cnt = %eval ( &all_error_cnt. + &error_cnt.);
    %rc_validate_institution_data(new_records=fincen_institution,
                                  record_type=filer,
                                  key_variable=institution_key,
                                  form_type=SARX,
                                  form_config_key=311,
                                  form_type_code=);
    %let all_error_cnt = %eval ( &all_error_cnt. + &error_cnt.);
    %if (&trans_rc.=0 and &all_error_cnt= 0) or &forceReplaceYN.=Y %then %do;
	    proc sql;
           delete from rc_cfg.fincen_institution(%fcf_get_ds_opts(lib_name=rc_cfg, ds_name=fincen_institution));
        quit;
        proc append base=rc_cfg.fincen_institution(%fcf_get_ds_opts(lib_name=rc_cfg, ds_name=fincen_institution, output_ds=Y))
                    data=fincen_institution force;
        run;
		%end;
     
%end;



/**********************************************************************************/
/* Sample branch data.                                                            */
/* This example shows how to integrate FSC_BRANCH_DIM data with institution       */
/* data that is associated with the branch using parent entity value.             */
/*                                                                                */
/* The parent_entity_number is used to join the data.                             */
/* There should be some logic to associate a branch with its parent institution.  */
/* Same list of branches for SAR and CTR but different fields are used for        */
/* different forms.                                                               */
/**********************************************************************************/

proc sql noprint;
    create table branch_dim as
    select  
          dim.branch_key, 
		  dim.branch_number length=100,
          dim.branch_name length=100,         
          dim.street_address_1,
          dim.street_address_2,
          dim.street_city_name,
          dim.street_state_code,
          dim.street_postal_code,
          dim.street_country_code
 
     from seg_kc.fsc_branch_dim as dim
     where dim.branch_key > 0
     ;
quit;

data branch_mod;
     if 0 then set rc_cfg.fincen_branch(%fcf_get_ds_opts(lib_name=rc_cfg, ds_name=fincen_branch));
     set branch_dim;
	 
	 /* Sample logic to associate a branch to a parent entity (institution)      */
	 /* will use this parent entity to populate institution information into     */
	 /* each branch.                                                             */
	 if branch_number < "1020000" then 
	    PARENT_ENTITY_NUMBER="111222333";
     else if branch_number < '1026080' then 
	    PARENT_ENTITY_NUMBER="222111333";
     else if branch_number < '1027000' then 
	    PARENT_ENTITY_NUMBER="333111222";
	 else if branch_number < '2010669' then 
	    PARENT_ENTITY_NUMBER="444111222";
	 else if branch_number < '2014992' then 
	    PARENT_ENTITY_NUMBER="555111222";
	 else 
	    PARENT_ENTITY_NUMBER="626111222";
	 
	 /* Sample to associate/generate an RSSD ID for each branch */
	 BRANCH_RSSD_ID = branch_number;
	 
	 /* drop the following columns that will populate from Parent Entity Institution. */
	 drop EFFECTIVE_FROM_DATE EFFECTIVE_TO_DATE BRANCH_ID
	      BRANCH_TAX_ID BRANCH_ID_TYPE_CODE BRANCH_PRIM_FED_REG_CODE
	      BRANCH_TYPE_CTR_CODE BRANCH_TYPE_CTR_OTHER
	      BRANCH_GAMING_TYPE_CODE BRANCH_GAMING_TYPE_OTHER      
	 ;
    
run;

/* Following Branch Fields will be populated from the Parent Entity Institution Values: */
/*     
       BRANCH_ID
	   BRANCH_ID_TYPE_CODE   
	   BRANCH_TAX_ID
       BRANCH_GAMING_TYPE_CODE
	   BRANCH_GAMING_TYPE_OTHER
	   BRANCH_PRIM_FED_REG_CODE
	   BRANCH_TYPE_CTR_CODE
	   BRANCH_TYPE_CTR_OTHER
	   EFFECTIVE_FROM_DATE
	   EFFECTIVE_TO_DATE 
*/
proc sql;
    create table branch_combined as
    select branch.*
          , inst.INSTITUTION_TYPE_CODE         as BRANCH_TYPE_CTR_CODE
		  , inst.INSTITUTION_TYPE_OTHER        as BRANCH_TYPE_CTR_OTHER
          , inst.INSTITUTION_GAMING_TYPE_CODE  as BRANCH_GAMING_TYPE_CODE        
          , inst.INSTITUTION_GAMING_TYPE_OTHER as BRANCH_GAMING_TYPE_OTHER
          , inst.INSTITUTION_PRIM_FED_REG_CODE as BRANCH_PRIM_FED_REG_CODE
		  , inst.INSTITUTION_ID                as BRANCH_ID 
          , inst.INSTITUTION_TAX_ID            as BRANCH_TAX_ID
		  , inst.INSTITUTION_ID_TYPE_CODE      as BRANCH_ID_TYPE_CODE
          , inst.EFFECTIVE_FROM_DATE
		  , inst.EFFECTIVE_TO_DATE
 
     from branch_mod as branch left join 
          fincen_institution as inst
     on branch.parent_entity_number=inst.entity_number
	    and inst.form_config_key=311     /* Will use the SARX defined Institution Information from Parent */
     ;
quit;

/* Final process of Branch data to clean up addresses, country codes and defining flags. */
data fincen_branch;
     if 0 then set rc_cfg.fincen_branch(%fcf_get_ds_opts(lib_name=rc_cfg, ds_name=fincen_branch));
     set branch_combined; 

     /* Required fields  */
     entity_number=branch_number;
     entity_name=branch_name;

     /* clean up branch data */ 
	 
	 /* Set the Branch Tax ID unknown flag it ID is blank */    
     if branch_tax_id = '' then 
	    branch_tax_id_unknown_flag='X';
     else 
	    branch_tax_id_unknown_flag='';

     /* fixing branch country code */      
     if street_address_1 ne '$' then
     do;
         if street_address_2 ne '$' then branch_address=strip(street_address_1)||' '||strip(street_address_2);
         else branch_address=street_address_1;
     end;
     if street_city_name ne '$' then branch_city_name=street_city_name;
     else branch_city_name='';
     if street_state_code ne '$' then branch_state_code=street_state_code;
     else branch_state_code='';
     if street_postal_code ne '$' then branch_zip_code=street_postal_code;
     else branch_zip_code='';
     
     /* convert 3 chars country code to 2 chars country code */
     branch_country_code=put(street_country_code,$_ctry_cd_3to2_fmt.);
     /* If branch state code is a U.S. terroritory, country code must be the same as state code */
     if branch_state_code in ('AS','FM','GU','MH','MP','PR','PW','VI','UM') then
        branch_country_code =  branch_state_code;

     /* blank out state code when the country is not in North America */
     if branch_country_code not in ('AS','FM','GU','MH','MP','PR','PW','VI','UM', 'US','CA', 'MX') then
        branch_state_code=''; 
    
     /* drop the final list of variables that were from SEG_KC and modified to fit branch table. */  
     drop 
          street_address_1 street_address_2  street_city_name street_state_code
          street_postal_code street_country_code;
    ;
run;



%if &rc_debug = N %then
    %do;
    proc sql;
         delete from rc_cfg.fincen_branch(%fcf_get_ds_opts(lib_name=rc_cfg, ds_name=fincen_branch));
    quit;
    proc append base=rc_cfg.fincen_branch(%fcf_get_ds_opts(lib_name=rc_cfg, ds_name=fincen_branch, output_ds=Y))
                data=fincen_branch  force;
    run; 
    %end;
 

%mend;

%macro rc_conversion_fmt;
 data country_fmt;
           set db_core.fsc_country_dim (keep=country_code_3 country_code_2 ) end=eof;
           retain fmtname '_ctry_cd_3to2_fmt'
                   type    'C'
                   ;
           length start end label $40; 
           start=country_code_3;
           end=start;
           label=country_code_2;
           if start='$' then label='';
           output;
           if eof then 
           do;
              start='**OTHER**';
              end='';
              hlo='O';
              label='';
              output;
            end; 
         run;
     
         proc sort data=country_fmt nodupkey;
         by start;
         run;
     
     
         proc format cntlin=country_fmt ;
    run;

%mend;

%rc_conversion_fmt;

%rc_load_sample_location;

%rc_refresh_entity_info;



